@@ -4,10 +4,12 @@ package DBIx::SearchBuilder;
 use strict;
 use warnings;
 
-our $VERSION = "1.59_01";
+our $VERSION = "1.63_01";
 
 use Clone qw();
 use Encode qw();
+use Scalar::Util qw(blessed);
+use DBIx::SearchBuilder::Util qw/ sorted_values /;
 
 =head1 NAME
 
@@ -134,6 +136,7 @@ sub CleanSlate {
     $self->{'first_row'}        = 0;
     $self->{'must_redo_search'} = 1;
     $self->{'show_rows'}        = 0;
+    $self->{'joins_are_distinct'} = 0;
     @{ $self->{'aliases'} } = ();
 
     delete $self->{$_} for qw(
@@ -437,7 +440,7 @@ sub BuildSelectQuery {
         $QueryString .= $clause;
         $QueryString .= $self->_OrderClause;
     }
-    elsif ($self->_isJoined) {
+    elsif ( !$self->{'joins_are_distinct'} && $self->_isJoined ) {
         $self->_DistinctQuery(\$QueryString);
     }
     else {
@@ -759,6 +762,17 @@ ENDSWITH is like LIKE, except it prepends a % to the beginning of the string
 MATCHES is equivalent to the database's LIKE -- that is, it's actually LIKE, but
 doesn't surround the string in % signs as LIKE does.
 
+=item "IN" and "NOT IN"
+
+VALUE can be an array reference or an object inherited from this class. If
+it's not then it's treated as any other operator and in most cases SQL would
+be wrong. Values in array are considered as constants and quoted according
+to QUOTEVALUE.
+
+If object is passed as VALUE then its select statement is used. If no L</Column>
+is selected then C<id> is used, if more than one selected then warning is issued
+and first column is used.
+
 =back
 
 =item ENTRYAGGREGATOR 
@@ -835,6 +849,33 @@ sub Limit {
         elsif ( $args{'OPERATOR'} =~ /ENDSWITH/i ) {
             $args{'VALUE'}    = "%" . $args{'VALUE'};
         }
+        elsif ( $args{'OPERATOR'} =~ /\bIN$/i ) {
+            if ( blessed $args{'VALUE'} && $args{'VALUE'}->isa(__PACKAGE__) ) {
+                # if no columns selected then select id
+                local $args{'VALUE'}{'columns'} = $args{'VALUE'}{'columns'};
+                unless ( $args{'VALUE'}{'columns'} ) {
+                    $args{'VALUE'}->Column( FIELD => 'id' );
+                } elsif ( @{ $args{'VALUE'}{'columns'} } > 1 ) {
+                    warn "Collection in '$args{OPERATOR}' with more than one column selected, using first";
+                    splice @{ $args{'VALUE'}{'columns'} }, 1;
+                }
+                $args{'VALUE'} = '('. $args{'VALUE'}->BuildSelectQuery .')';
+                $args{'QUOTEVALUE'} = 0;
+            }
+            elsif ( ref $args{'VALUE'} ) {
+                if ( $args{'QUOTEVALUE'} ) {
+                    my $dbh = $self->_Handle->dbh;
+                    $args{'VALUE'} = join ', ', map $dbh->quote( $_ ), @{ $args{'VALUE'} };
+                } else {
+                    $args{'VALUE'} = join ', ', @{ $args{'VALUE'} };
+                }
+                $args{'VALUE'} = "($args{VALUE})";
+                $args{'QUOTEVALUE'} = 0;
+            }
+            else {
+                # otherwise behave in backwards compatible way
+            }
+        }
         $args{'OPERATOR'} =~ s/(?:MATCHES|ENDSWITH|STARTSWITH)/LIKE/i;
 
         #if we're explicitly told not to to quote the value or
@@ -1005,7 +1046,7 @@ sub _WhereClause {
     #Go through all restriction types. Build the where clause from the
     #Various subclauses.
     my $where_clause = '';
-    foreach my $subclause ( grep $_, values %{ $self->{'subclauses'} } ) {
+    foreach my $subclause ( grep $_, sorted_values($self->{'subclauses'}) ) {
         $where_clause .= " AND " if $where_clause;
         $where_clause .= $subclause;
     }
@@ -1023,7 +1064,7 @@ sub _CompileGenericRestrictions {
 
     my $result = '';
     #Go through all the restrictions of this type. Buld up the generic subclause
-    foreach my $restriction ( grep @$_, values %{ $self->{'restrictions'} } ) {
+    foreach my $restriction ( grep @$_, sorted_values($self->{'restrictions'}) ) {
         $result .= " AND " if $result;
         $result .= '(';
         foreach my $entry ( @$restriction ) {
@@ -1336,7 +1377,7 @@ sub FirstPage {
 =head3 GotoPage
 
 Takes an integer number and jumps to that page or first page if
-number ommitted. Numbering starts from zero.
+number omitted. Numbering starts from zero.
 
 =cut
 
@@ -1538,6 +1579,13 @@ Otherwise, the function is inserted verbatim, with no substitution.
 
 =back
 
+=item AS
+
+The B<column> alias to use instead of the default.  The default column alias is
+either the column's name (i.e. what is passed to FIELD) if it is in this table
+(ALIAS is 'main') or an autogenerated alias.  Pass C<undef> to skip column
+aliasing entirely.
+
 =back
 
 If a FIELD is provided and it is in this table (ALIAS is 'main'), then
@@ -1558,6 +1606,9 @@ column's result:
     my $article = $articles->First;
     my $now = $article->_Value( $time_alias );
 
+To choose the column's alias yourself, pass a value for the AS parameter (see
+above).  Be careful not to conflict with existing column aliases.
+
 =cut
 
 sub Column {
@@ -1599,25 +1650,28 @@ sub Column {
         $name = 'NULL';
     }
 
-    my $column;
-    if (
-        $args{FIELD} && $args{ALIAS} eq 'main'
-        && (!$args{'TABLE'} || $args{'TABLE'} eq $self->Table )
-    ) {
-        $column = $args{FIELD};
+    my $column = $args{'AS'};
 
-        # make sure we don't fetch columns with duplicate aliases
-        if ( $self->{columns} ) {
-            my $suffix = " AS \L$column";
-            if ( grep index($_, $suffix, -length $suffix) >= 0, @{ $self->{columns} } ) {
-                $column .= scalar @{ $self->{columns} };
+    if (not defined $column and not exists $args{'AS'}) {
+        if (
+            $args{FIELD} && $args{ALIAS} eq 'main'
+            && (!$args{'TABLE'} || $args{'TABLE'} eq $self->Table )
+        ) {
+            $column = $args{FIELD};
+
+            # make sure we don't fetch columns with duplicate aliases
+            if ( $self->{columns} ) {
+                my $suffix = " AS \L$column";
+                if ( grep index($_, $suffix, -length $suffix) >= 0, @{ $self->{columns} } ) {
+                    $column .= scalar @{ $self->{columns} };
+                }
             }
         }
+        else {
+            $column = "col" . @{ $self->{columns} ||= [] };
+        }
     }
-    else {
-        $column = "col" . @{ $self->{columns} ||= [] };
-    }
-    push @{ $self->{columns} ||= [] }, "$name AS \L$column";
+    push @{ $self->{columns} ||= [] }, defined($column) ? "$name AS \L$column" : $name;
     return $column;
 }
 
@@ -1635,6 +1689,23 @@ sub Columns {
     $self->Column( FIELD => $_ ) for @_;
 }
 
+=head2 AdditionalColumn
+
+Calls L</Column>, but first ensures that this table's standard columns are
+selected as well.  Thus, each call to this method results in an additional
+column selected instead of replacing the default columns.
+
+Takes a hash of parameters which is the same as L</Column>.  Returns the result
+of calling L</Column>.
+
+=cut
+
+sub AdditionalColumn {
+    my $self = shift;
+    $self->Column( FUNCTION => "main.*", AS => undef )
+        unless grep { /^\Qmain.*\E$/ } @{$self->{columns}};
+    return $self->Column(@_);
+}
 
 =head2 Fields TABLE
 
