@@ -9,6 +9,8 @@ use DBI;
 use Class::ReturnValue;
 use Encode qw();
 
+use DBIx::SearchBuilder::Util qw/ sorted_values /;
+
 use vars qw(@ISA %DBIHandle $PrevHandle $DEBUG %TRANSDEPTH %FIELDS_IN_TABLE);
 
 
@@ -357,7 +359,7 @@ Splits the key value pairs, constructs an INSERT statement
 and returns query string and set of bind values.
 
 This method is more useful for subclassing in DB specific
-handles. L</Insert> method is prefered for end users.
+handles. L</Insert> method is preferred for end users.
 
 =cut
 
@@ -377,6 +379,35 @@ sub InsertQueryString {
     return ($QueryString, @bind);
 }
 
+=head2 InsertFromSelect
+
+Takes table name, array reference with columns, select query
+and list of bind values. Inserts data select by the query
+into the table.
+
+To make sure call is portable every column in result of
+the query should have unique name or should be aliased.
+See L<DBIx::SearchBuilder::Handle::Oracle/InsertFromSelect> for
+details.
+
+=cut
+
+sub InsertFromSelect {
+    my ($self, $table, $columns, $query, @binds) = @_;
+
+    $columns = join ', ', @$columns
+        if $columns;
+
+    my $full_query = "INSERT INTO $table";
+    $full_query .= " ($columns)" if $columns;
+    $full_query .= ' '. $query;
+    my $sth = $self->SimpleQuery( $full_query, @binds );
+    return $sth unless $sth;
+
+    my $rows = $sth->rows;
+    return $rows == 0? '0E0' : $rows;
+}
+
 =head2 UpdateRecordValue 
 
 Takes a hash with fields: Table, Column, Value PrimaryKeys, and 
@@ -412,7 +443,7 @@ sub UpdateRecordValue {
 
   ## Constructs the where clause.
   my $where  = 'WHERE ';
-  foreach my $key (keys %{$args{'PrimaryKeys'}}) {
+  foreach my $key (sort keys %{$args{'PrimaryKeys'}}) {
      $where .= $key . "=?" . " AND ";
      push (@bind, $args{'PrimaryKeys'}{$key});
   }
@@ -446,6 +477,62 @@ sub UpdateTableValue  {
     return $self->UpdateRecordValue(%args)
 }
 
+=head1 SimpleUpdateFromSelect
+
+Takes table name, hash reference with (column, value) pairs,
+select query and list of bind values.
+
+Updates the table, but only records with IDs returned by the
+selected query, eg:
+
+    UPDATE $table SET %values WHERE id IN ( $query )
+
+It's simple as values are static and search only allowed
+by id.
+
+=cut
+
+sub SimpleUpdateFromSelect {
+    my ($self, $table, $values, $query, @query_binds) = @_;
+
+    my @columns; my @binds;
+    for my $k (sort keys %$values) {
+        push @columns, $k;
+        push @binds, $values->{$k};
+    }
+
+    my $full_query = "UPDATE $table SET ";
+    $full_query .= join ', ', map "$_ = ?", @columns;
+    $full_query .= ' WHERE id IN ('. $query .')';
+    my $sth = $self->SimpleQuery( $full_query, @binds, @query_binds );
+    return $sth unless $sth;
+
+    my $rows = $sth->rows;
+    return $rows == 0? '0E0' : $rows;
+}
+
+=head1 DeleteFromSelect
+
+Takes table name, select query and list of bind values.
+
+Deletes from the table, but only records with IDs returned by the
+select query, eg:
+
+    DELETE FROM $table WHERE id IN ($query)
+
+=cut
+
+sub DeleteFromSelect {
+    my ($self, $table, $query, @binds) = @_;
+    my $sth = $self->SimpleQuery(
+        "DELETE FROM $table WHERE id IN ($query)",
+        @binds
+    );
+    return $sth unless $sth;
+
+    my $rows = $sth->rows;
+    return $rows == 0? '0E0' : $rows;
+}
 
 =head2 SimpleQuery QUERY_STRING, [ BIND_VALUE, ... ]
 
@@ -958,7 +1045,7 @@ sub Join {
         push @{$args{SearchBuilder}{aliases}}, @{$collection->{aliases}};
 
         # Move over joins, as well
-        for my $join (keys %{$collection->{left_joins}}) {
+        for my $join (sort keys %{$collection->{left_joins}}) {
             my %alias = %{$collection->{left_joins}{$join}};
             $alias{depends_on} = $alias if $alias{depends_on} eq "main";
             $alias{criteria} = $self->_RenameRestriction(
@@ -1084,7 +1171,7 @@ sub _BuildJoins {
     while ( my @list =
         grep !$processed{ $_ }
             && (!$joins->{ $_ }{'depends_on'} || $processed{ $joins->{ $_ }{'depends_on'} }),
-        keys %$joins
+        sort keys %$joins
     ) {
         foreach my $join ( @list ) {
             $processed{ $join }++;
@@ -1098,7 +1185,7 @@ sub _BuildJoins {
                         $_->{'field'} .' '. $_->{'op'} .' '. $_->{'value'}:
                         $_
                 }
-                map { ('(', @$_, ')', $aggregator) } values %{ $meta->{'criteria'} };
+                map { ('(', @$_, ')', $aggregator) } sorted_values($meta->{'criteria'});
             pop @tmp;
             $join_clause .= join ' ', @tmp;
         }
@@ -1128,7 +1215,7 @@ sub OptimizeJoins {
     # finally we'll get ordered list with leafes in the beginning and top most nodes at
     # the end.
     while ( my @list = grep !$processed{ $_ }
-            && $processed{ $joins->{ $_ }{'depends_on'} }, keys %$joins )
+            && $processed{ $joins->{ $_ }{'depends_on'} }, sort keys %$joins )
     {
         unshift @ordered, @list;
         $processed{ $_ }++ foreach @list;
@@ -1168,19 +1255,19 @@ sub MayBeNull {
 
     # build full list of generic conditions
     my @conditions;
-    foreach ( grep @$_, values %{ $args{'SearchBuilder'}->{'restrictions'} } ) {
+    foreach ( grep @$_, sorted_values($args{'SearchBuilder'}->{'restrictions'}) ) {
         push @conditions, 'AND' if @conditions;
         push @conditions, '(', @$_, ')';
     }
 
     # find tables that depends on this alias and add their join conditions
-    foreach my $join ( values %{ $args{'SearchBuilder'}->{'left_joins'} } ) {
+    foreach my $join ( sorted_values($args{'SearchBuilder'}->{'left_joins'}) ) {
         # left joins on the left side so later we'll get 1 AND x expression
         # which equal to x, so we just skip it
         next if $join->{'type'} eq 'LEFT';
         next unless $join->{'depends_on'} eq $args{'ALIAS'};
 
-        my @tmp = map { ('(', @$_, ')', $join->{'entry_aggregator'}) } values %{ $join->{'criteria'} };
+        my @tmp = map { ('(', @$_, ')', $join->{'entry_aggregator'}) } sorted_values($join->{'criteria'});
         pop @tmp;
 
         @conditions = ('(', @conditions, ')', 'AND', '(', @tmp ,')');
@@ -1354,6 +1441,159 @@ sub Log {
 
 }
 
+=head2 SimpleDateTimeFunctions
+
+See L</DateTimeFunction> for details on supported functions.
+This method is for implementers of custom DB connectors.
+
+Returns hash reference with (function name, sql template) pairs.
+
+=cut
+
+sub SimpleDateTimeFunctions {
+    my $self = shift;
+    return {
+        datetime       => 'SUBSTR(?, 1,  19)',
+        time           => 'SUBSTR(?, 12,  8)',
+
+        hourly         => 'SUBSTR(?, 1,  13)',
+        hour           => 'SUBSTR(?, 12, 2 )',
+
+        date           => 'SUBSTR(?, 1,  10)',
+        daily          => 'SUBSTR(?, 1,  10)',
+
+        day            => 'SUBSTR(?, 9,  2 )',
+        dayofmonth     => 'SUBSTR(?, 9,  2 )',
+
+        monthly        => 'SUBSTR(?, 1,  7 )',
+        month          => 'SUBSTR(?, 6,  2 )',
+
+        annually       => 'SUBSTR(?, 1,  4 )',
+        year           => 'SUBSTR(?, 1,  4 )',
+    };
+}
+
+=head2 DateTimeFunction
+
+Takes named arguments:
+
+=over 4
+
+=item * Field - SQL expression date/time function should be applied
+to. Note that this argument is used as is without any kind of quoting.
+
+=item * Type - name of the function, see supported values below.
+
+=item * Timezone - optional hash reference with From and To values,
+see L</ConvertTimezoneFunction> for details.
+
+=back
+
+Returns SQL statement. Returns NULL if function is not supported.
+
+=head3 Supported functions
+
+Type value in L</DateTimeFunction> is case insesitive. Spaces,
+underscores and dashes are ignored. So 'date time', 'DateTime'
+and 'date_time' are all synonyms. The following functions are
+supported:
+
+=over 4
+
+=item * date time - as is, no conversion, except applying timezone
+conversion if it's provided.
+
+=item * time - time only
+
+=item * hourly - datetime prefix up to the hours, e.g. '2010-03-25 16'
+
+=item * hour - hour, 0 - 23
+
+=item * date - date only
+
+=item * daily - synonym for date
+
+=item * day of week - 0 - 6, 0 - Sunday
+
+=item * day - day of month, 1 - 31
+
+=item * day of month - synonym for day
+
+=item * day of year - 1 - 366, support is database dependent
+
+=item * month - 1 - 12
+
+=item * monthly - year and month prefix, e.g. '2010-11'
+
+=item * year - e.g. '2023'
+
+=item * annually - synonym for year
+
+=item * week of year - 0-53, presence of zero week, 1st week meaning
+and whether week starts on Monday or Sunday heavily depends on database.
+
+=back
+
+=cut
+
+sub DateTimeFunction {
+    my $self = shift;
+    my %args = (
+        Field => undef,
+        Type => '',
+        Timezone => undef,
+        @_
+    );
+
+    my $res = $args{'Field'} || '?';
+    if ( $args{'Timezone'} ) {
+        $res = $self->ConvertTimezoneFunction(
+            %{ $args{'Timezone'} },
+            Field => $res,
+        );
+    }
+
+    my $norm_type = lc $args{'Type'};
+    $norm_type =~ s/[ _-]//g;
+    if ( my $template = $self->SimpleDateTimeFunctions->{ $norm_type } ) {
+        $template =~ s/\?/$res/;
+        $res = $template;
+    }
+    else {
+        return 'NULL';
+    }
+    return $res;
+}
+
+=head2 ConvertTimezoneFunction
+
+Generates a function applied to Field argument that converts timezone.
+By default converts from UTC. Examples:
+
+    # UTC => Moscow
+    $handle->ConvertTimezoneFunction( Field => '?', To => 'Europe/Moscow');
+
+If there is problem with arguments or timezones are equal
+then Field returned without any function applied. Field argument
+is not escaped in any way, it's your job.
+
+Implementation is very database specific. To be portable convert
+from UTC or to UTC. Some databases have internal storage for
+information about timezones that should be kept up to date.
+Read documentation for your DB.
+
+=cut
+
+sub ConvertTimezoneFunction {
+    my $self = shift;
+    my %args = (
+        From  => 'UTC',
+        To    => undef,
+        Field => '',
+        @_
+    );
+    return $args{'Field'};
+}
 
 
 =head2 DESTROY
